@@ -10,11 +10,11 @@ Dancer::Plugin::Database::Core - Shared core for D1 and D2 Database plugins
 
 =head1 VERSION
 
-Version 0.08
+Version 0.02
 
 =cut
 
-our $VERSION = '0.08_03';
+our $VERSION = '0.02';
 
 my %handles;
 # Hashref used as key for default handle, so we don't have a magic value that
@@ -94,39 +94,36 @@ sub database {
                 $logger->(debug => "Database connection went away, reconnecting");
                 $hook_exec->('database_connection_lost', $handle->{dbh});
 
-                if ($handle->{dbh}) {
-                    eval { $handle->{dbh}->disconnect }
-                }
-
-                # Need a new handle.
-                # Fall through to the new connection codepath to get one.
+                if ($handle->{dbh}) { eval { $handle->{dbh}->disconnect } }
+                return _get_connection($conn_details, $logger, $hook_exec);
             }
         }
-    }
-
-    # Get a new connection
-    $handle->{dbh} = _get_connection($conn_details, $logger, $hook_exec);
-
-    if ($handle->{dbh}) {
-
-        $handle->{last_connection_check} = time;
-        $handles{$pid_tid}{$handle_key} = $handle;
-
-        if (ref $handle_key && ref $handle_key ne ref $def_handle) {
-            # We were given a hashref of connection settings.  Shove a
-            # reference to that hashref into the handle, so that the hashref
-            # doesn't go out of scope for the life of the handle.
-            # Otherwise, that area of memory could be re-used, and, given
-            # different DB settings in a hashref that just happens to have
-            # the same address, we'll happily hand back the original handle.
-            # See http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=665221
-            # Thanks to Sam Kington for suggesting this fix :)
-            $handle->{_orig_settings_hashref} = $handle_key;
-        }
-
-        return ($handle->{dbh}, $settings);
     } else {
-        return (undef, $settings);
+
+        # Get a new connection
+        ($handle->{dbh}, $settings) = _get_connection($conn_details, $logger, $hook_exec);
+
+        if ($handle->{dbh}) {
+
+            $handle->{last_connection_check} = time;
+            $handles{$pid_tid}{$handle_key} = $handle;
+
+            if (ref $handle_key && ref $handle_key ne ref $def_handle) {
+                # We were given a hashref of connection settings.  Shove a
+                # reference to that hashref into the handle, so that the hashref
+                # doesn't go out of scope for the life of the handle.
+                # Otherwise, that area of memory could be re-used, and, given
+                # different DB settings in a hashref that just happens to have
+                # the same address, we'll happily hand back the original handle.
+                # See http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=665221
+                # Thanks to Sam Kington for suggesting this fix :)
+                $handle->{_orig_settings_hashref} = $handle_key;
+            }
+
+            return ($handle->{dbh}, $settings);
+        } else {
+            return (undef, $settings);
+        }
     }
 
 }
@@ -198,7 +195,7 @@ sub _get_connection {
     my $driver = '';
     if ($settings->{dsn}) {
         $dsn = $settings->{dsn};
-        ($driver) = $dsn =~ m{^dbi:([^:]+)}i;
+        ($driver) = $dsn =~ m{dbi:([^:]+)};
     } else {
         $dsn = "dbi:" . $settings->{driver};
         $driver = $settings->{driver};
@@ -217,16 +214,11 @@ sub _get_connection {
             $settings->{dbname} = delete $settings->{database};
         }
 
-        for (qw(database dbname host port sid server)) {
+        for (qw(database dbname host port sid)) {
             if (exists $settings->{$_}) {
                 push @extra_args, $_ . "=" . $settings->{$_};
             }
         }
-        if (my $even_more_dsn_args = $settings->{dsn_extra}) {
-            foreach my $arg ( keys %$even_more_dsn_args ) {
-                push @extra_args, $arg . '=' . $even_more_dsn_args->{$arg};
-            }
-        }
         $dsn .= ':' . join(';', @extra_args) if @extra_args;
     }
 
@@ -268,7 +260,7 @@ sub _get_connection {
     if (!$dbh) {
         $logger->(error => "Database connection failed - " . $DBI::errstr);
         $hook_exec->('database_connection_failed', $settings);
-        return undef;
+        return (undef, $settings);
     } elsif (exists $settings->{on_connect_do}) {
         my $to_do = ref $settings->{on_connect_do} eq 'ARRAY'
             ?   $settings->{on_connect_do}
@@ -301,7 +293,7 @@ sub _get_connection {
     $package .= '.pm';
     require $package;
 
-    return bless($dbh => $handle_class);
+    return (bless($dbh => $handle_class), $settings);
 }
 
 
@@ -395,7 +387,7 @@ This license does not grant you the right to use any trademark, service
 mark, tradename, or logo of the Copyright Holder.
 
 This license includes the non-exclusive, worldwide, free-of-charge
-patent license to make, have made, use, er to sell, sell, import and
+patent license to make, have made, use, offer to sell, sell, import and
 otherwise transfer the Package with respect to any patent claims
 licensable by the Copyright Holder that are necessarily infringed by the
 Package. If you institute patent litigation (including a cross-claim or
